using System;
using System.Security.Cryptography;
using System.Text;
using Autofac;
using FitConnect;
using FitConnect.Encryption;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using MockContainer;
using NUnit.Framework;

namespace SenderTest;

public class JweTest {
    private ILogger<JweTest> _logger;
    private Sender _sender;
    private IContainer _container;
    private MockSettings _settings;


    [SetUp]
    public void SetUp() {
        _logger = LoggerFactory.Create(cfg => cfg.AddConsole())
            .CreateLogger<JweTest>();

        _sender = Sender.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            _logger);

        _container = MockContainer.Container.Create();
        _settings = _container.Resolve<MockSettings>();
    }


    [Test]
    public void TestJwe() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var dummyText = "Test plain text";
        var cypher = engine.Encrypt(dummyText);
        var plain = engine.Decrypt(cypher);


        // Assert
        plain.plainText.Should().Be(dummyText);
    }

    [Test]
    public void TestJwe_withBytes() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var plainBytes = RandomNumberGenerator.GetBytes(8192);
        var cypher = engine.Encrypt(plainBytes);
        var plain = engine.Decrypt(cypher);


        // Assert
        plain.plainBytes.Should().BeEquivalentTo(plainBytes);
    }

    [Test]
    public void TestJwe_withBytes2() {
        // Arrange
        var engine = new FitEncryption(_logger) {
            PublicKeyEncryption = _settings.PublicKeyEncryption,
            PrivateKeyDecryption = _settings.PrivateKeyDecryption
        };

        // Act
        var plainBytes = RandomNumberGenerator.GetBytes(512);
        var cypher = engine.Encrypt(Convert.ToBase64String(plainBytes));
        var plain = engine.Decrypt(cypher);
        
        // Assert
        plain.plainText.Should().BeEquivalentTo(Convert.ToBase64String(plainBytes));
    }
}
