using System;
using System.Text;
using FitConnect;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using NUnit.Framework;

namespace SenderTest;

[Ignore("Encryption strategy does not match the one used on server")]
public class SenderEncryptionWithoutCertificateTest {
    private const string ToEncrypt = "Hello World";
    private string _cypherText = null!;
    private ILogger<SenderEncryptionWithoutCertificateTest> _logger = null!;
    private Sender _sender = null!;

    /*
     * Encryption test must be changed for production to only allow extern signed certificates
     * and forbid self-signed certificates.
     */

    [OneTimeSetUp]
    public void OneTimeSetUp() {
        _logger = LoggerFactory.Create(cfg => cfg.AddConsole())
            .CreateLogger<SenderEncryptionWithoutCertificateTest>();
        _sender = Sender.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            _logger);
    }


    [Test]
    [Order(10)]
    public void EncryptData_ShouldNotThrowAnyException() {
        var cypher = _sender.Encryption.Encrypt(Encoding.UTF8.GetBytes(ToEncrypt));
        _logger.LogInformation("Cypher: {}", cypher);
    }


    [Test]
    [Order(20)]
    public void DecryptData_ShouldMatchToEncrypt() {
        var (plain,_, tag) = _sender.Encryption.Decrypt(_cypherText);

        plain.Should().Be(ToEncrypt);
    }
}
